/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <dirent.h>
#include <stdarg.h>


#include "main.h"
#include "config.h"
#include "misc.h"
#include "fs.h"
#include "strings.h"



int ovcp_write(int socket_fd, char *data, int len, void *session)
{
        int retval;

        #ifdef HAVE_LIBGNUTLS        
        if(session != NULL)

                        gnutls_record_send (*((gnutls_session_t *)session), data, len);
        else
        #endif
                write(socket_fd, data, len);

}

int ovcp_read(int socket_fd, char *data, int len, void *session)
{
        int retval;
        
        #ifdef HAVE_LIBGNUTLS
        if(session != NULL)
                        gnutls_record_recv (*((gnutls_session_t *)session), data, len);
        else
        #endif
                read(socket_fd, data, len);

}

char *ovcp_readline(int fd, void *session)
{
	int i = 0;
	int ret;
	char c = 0;
	char *buffer = NULL;

	while(i < 15000)
	{
                #ifdef HAVE_LIBGNUTLS
                if(session != NULL)
                      ret = gnutls_record_recv(*((gnutls_session_t *)session), &c, 1);     
                else
                #endif
		      ret = read(fd, &c, 1);

		if(ret != 1 || c == '\n' || c == '\r')
			break;

		buffer = realloc(buffer, i+2);
		buffer[i] = c;

		i++;
	}

	if(buffer != NULL)
		buffer[i] = 0;

	return buffer;
}

int ovcp_close(int socket_fd, void *session)
{

        #ifdef HAVE_LIBGNUTLS

        if(session != NULL)
        {
                gnutls_bye (*((gnutls_session_t *)session), GNUTLS_SHUT_WR);
                gnutls_deinit (*((gnutls_session_t *)session));
                free(session);
        }

        #endif

        close(socket_fd);
}

int vserver_xid(char *name)
{
        char *path, *context_str;
        int xid = 0;

        path = concat(global_settings.config_dir, "/", name, "/context", NULL);

        context_str = file_read(path);

        if(context_str != NULL)
        {
                trim_string(context_str);
                xid = atoi(context_str);

                free(context_str);
        }

        free(path);

        return xid;
}

int vserver_exist(char *name)
{

        char *path;

        path = concat(global_settings.config_dir, "/", name, NULL);

        DIR *dirst = opendir(path);

        free(path);

        if(dirst != NULL)
        {
                closedir(dirst);
                return 1;
        }


        return 0;

}
int vserver_lock(char *name)
{
        char *path;

        path = concat(global_settings.config_dir, "/", name, "/locked", NULL);

        if(!file_exist(path))
        {
                file_touch(path);
                free(path);
        }
        else
        {
                free(path);
                return -1;
        }

        return 1;
}


int vserver_unlock(char *name)
{
        char *path;

        path = concat(global_settings.config_dir, "/", name, "/locked", NULL);

        unlink(path);
        free(path);

        return 1;
}


char *exec_cmd(char *cmd, int exectime, ...)
{
        char *output;

        char **argv = NULL, *next_arg;
        va_list ap;
        int i = 0;

        va_start(ap, exectime);


        while((next_arg = va_arg(ap, char *)) != NULL)
        {
                argv = (char **)realloc(argv, sizeof(char **) * (i+2));

                argv[i] = next_arg;

                i++;
        }

        argv[i] = NULL;

        va_end(ap);

        output = exec_cmdv(cmd, exectime, argv);

        free(argv);

        return output;
}


char *exec_cmdv(char *cmd, int exectime, char **argv)
{

        int pipe_fds[2];
        char *output = NULL;
        int res, len = 0, ges = 0;

        struct timeval timeout;
        fd_set fdset;



        pipe(pipe_fds);

        if(fork() == 0)
        {
                close(1);
                close(2);
                dup(pipe_fds[1]);
                dup(pipe_fds[1]);
                close(pipe_fds[0]);
                close(pipe_fds[1]);

                execvp(cmd, argv);

                perror("execvp");
                exit(-1);
        }
        close(pipe_fds[1]);

        do
        {
                len = 0;

                FD_ZERO(&fdset);
                FD_SET(pipe_fds[0], &fdset);

                timeout.tv_sec = exectime;
                timeout.tv_usec = 0;


                res = select(pipe_fds[0]+1, &fdset, NULL, NULL, &timeout);

                if(res != 0 && res != -1)
                {
                        output = (char *)realloc(output, ges+1000+1);
                        len = read(pipe_fds[0], output+ges, 1000);

                        ges += len;
                        output[ges] = 0;
                }
        }
        while(len != 0);

        close(pipe_fds[0]);

	if(output != NULL)
	{
		if(strlen(output) == 0)
		{
			free(output);
			output = NULL;
		}
		else
		{
        		output = (char *)realloc(output, ges+1);
		}
	}



        return output;
}




int die(char *string)
{
        printf("%s", string);
        exit(-1);
}


char **parse_ips(char *name, int *num)
{
        int counter = 0;
        struct dirent *dir_info;
        char *path, *ip;
        DIR *dirst;

        char **output = NULL;
        *num = 0;


        path = concat(global_settings.config_dir, "/", name,"/interfaces", NULL);
        dirst = opendir(path);
        free(path);

        if(dirst != NULL)
        {

                while ((dir_info = readdir(dirst)) != NULL)
                {

                        if(strcmp(dir_info->d_name, ".") != 0 && strcmp(dir_info->d_name, "..") != 0)
                        {

                                path = concat(global_settings.config_dir, "/", name,"/interfaces/", dir_info->d_name, NULL);

                                if(dir_exist(path))
                                {
                                        free(path);
                                        path = concat(global_settings.config_dir, "/", name,"/interfaces/", dir_info->d_name, "/ip", NULL);

                                        ip = file_read(path);
                                        if(ip != NULL)
                                        {
                                                trim_string(ip);

                                                output = realloc(output, (counter+1) * sizeof(char **));
                                                output[counter] = ip;

                                                #ifdef DEBUG
                                                        printf("IP %s:%d: %s\n", name, counter, output[counter]);
                                                #endif

                                                counter++;
                                        }
                                }

                                free(path);

                        }

                }

                *num = counter;
                closedir(dirst);
        }

        return output;
}

