/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>


#include "main.h"
#include "node.h"
#include "strings.h"
#include "fs.h"



char *node_get_interfaces()
{
	FILE *dev_file;
	char *output = NULL, *word = NULL;
	int i = 0, output_len = 0;
	
	dev_file = fopen("/proc/net/dev", "r");

	if(dev_file != NULL)
	{
		
		while((fscanf(dev_file, "%as", &word)) != EOF)
		{

			if(word != NULL)
			{
				i = 0;

				while(i != strlen(word))
				{
					if(word[i] == ':')
					{
						#ifdef DEBUG
							printf("FOUND DEV: %s\n", word);
						#endif

						word[i] = 0;
						output_len += strlen(word)+1;
						output = realloc(output, output_len+2);
						strcpy(output+output_len-strlen(word)-1, word);

						output[output_len-1] = ' ';
						output[output_len] = '\n';
						output[output_len+1] = 0;

						break;
					}
	
					i++;
				}

				free(word);
			}
		}

		free(word);
		

	}
	

	fclose(dev_file);
	return output;
}

char *node_get_images()
{
	DIR *image_dir;
 	struct dirent *entry;
	char *output = NULL;
	int output_len = 0;
	char *path;


	if((image_dir = opendir(global_settings.image_dir)) != NULL)
	{

		while((entry = readdir(image_dir)) != NULL)
		{

			if(entry->d_name[0] != '.')
			{

				path = concat(global_settings.image_dir, "/", entry->d_name, NULL);

				if(dir_exist(path))
				{
					#ifdef DEBUG
							printf("FOUND IMAGE: %s\n", entry->d_name);
					#endif

					output = realloc(output, output_len + strlen(entry->d_name)+3);
					strcpy(output+output_len, entry->d_name);

					output_len += strlen(entry->d_name)+1;
	
					output[output_len-1] = ' ';
					output[output_len] = '\n';
					output[output_len+1] = 0;
				}

				free(path);
			}
		}

		closedir(image_dir);
	}
	

	if(output == NULL)
	{
		output = strdup("ERROR: Image Directory not found or empty\n"); 
	}

	return output;
}

