/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "misc.h"
#include "parseconfig.h"
#include "strings.h"
#include "fs.h"

int parse_config(char *filename, struct config_st *config)
{
	int fd;
	unsigned int args;
	char *line, **argv;

	memset(config, 0, sizeof(struct config_st));


	if((fd = open(filename, O_RDONLY)) != -1)
	{

		while((line = readline(fd)) != NULL)
		{
			argv = split_string(line, '=', &args);
		
			if(args >= 2)
			{	


				if(strcmp(argv[0], "IP") == 0)
				{
					config->ip = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->ip, argv[1]);
				}
				else
				if(strcmp(argv[0], "Port") == 0)
				{
					config->port = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->port, argv[1]);
				}
				else
				if(strcmp(argv[0], "Password") == 0)
				{
					config->password = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->password, argv[1]);
				}
				if(strcmp(argv[0], "Logfile") == 0)
				{
					config->logfile = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->logfile, argv[1]);
				}
				if(strcmp(argv[0], "StartXID") == 0)
				{
					config->startxid = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->startxid, argv[1]);
				}
				else
				if(strcmp(argv[0], "RootDir") == 0)
				{
					config->root_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->root_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "ConfigDir") == 0)
				{
					config->config_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->config_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "BackupDir") == 0)
				{
					config->backup_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->backup_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "ImageDir") == 0)
				{
					config->image_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->image_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "RunDir") == 0)
				{
					config->run_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->run_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "IpfmDir") == 0)
				{
					config->ipfm_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->ipfm_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "CacheDir") == 0)
				{
					config->cache_dir = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->cache_dir, argv[1]);
				}
				else
				if(strcmp(argv[0], "PidFile") == 0)
				{
					config->pid_file = (char *) malloc(strlen(argv[1])+1);
					strcpy(config->pid_file, argv[1]);
				}

			
			}
	
			free_strings(argv, args);
			free(line);
		}


	}
	else
	{
		printf("\nERROR: Could not open %s\n\n", filename);
		exit(-1);
	}

	if(config->ip == NULL)
		die("ERROR: IP Directive not Found\n");
	if(config->port == NULL)
		die("ERROR: Port Directive not Found\n");
	if(config->password == NULL)
		die("ERROR: Password Directive not Found\n");
	if(config->logfile == NULL)
		die("ERROR: Logfile Directive not Found\n");
	if(config->startxid == NULL)
		die("ERROR: StartXID Directive not Found\n");
	if(config->root_dir == NULL || strlen(config->root_dir) <= 1)
		die("ERROR: RootDir Directive not Found\n");
	if(config->config_dir == NULL)
		die("ERROR: ConfigDir Directive not Found\n");
	if(config->backup_dir == NULL)
		die("ERROR: BackupDir Directive not Found\n");
	if(config->image_dir == NULL)
		die("ERROR: ImageDir Directive not Found\n");
	if(config->ipfm_dir == NULL)
		die("ERROR: IpfmDir Directive not Found\n");
	if(config->run_dir == NULL)
		die("ERROR: RunDir Directive not Found\n");
	if(config->cache_dir == NULL)
		die("ERROR: CacheDir Directive not Found\n");
	if(config->pid_file == NULL)
		die("ERROR: PidFile Directive not Found\n");
	
	if(!dir_exist(config->root_dir))
		die("ERROR: Root Directory does not exist\n");
	if(!dir_exist(config->config_dir))
		die("ERROR: Config Directory does not exist\n");
	if(!dir_exist(config->backup_dir))
		die("ERROR: Backup Directory does not exist\n");
	if(!dir_exist(config->image_dir))
		die("ERROR: Image Directory does not exist\n");
	if(!dir_exist(config->ipfm_dir))
		die("ERROR: Ipfm Directory does not exist\n");
	if(!dir_exist(config->run_dir))
		die("ERROR: Run Directory does not exist\n");


	#ifdef DEBUG
	
	printf("\nOptions:\n\nIP Address: %s\nPort Number: %s\nLogfile: %s\nStartXID: %s\nVServer Root: %s\nVServer Configs: %s\nVServer Backups: %s\nVServer Images: %s\nVServer Run Dir: %s\nIPFM Dir: %s\nVserver Cache Dir: %s\n\n",
		 config->ip, config->port, config->logfile, config->startxid, config->root_dir, config->config_dir, config->backup_dir, config->image_dir, config->run_dir, config->ipfm_dir, config->cache_dir);
	#endif

	close(fd);

	return 1;
}
