/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>

#include "main.h"
#include "misc.h"
#include "config.h"
#include "fs.h"
#include "strings.h"
#include "get.h"
#include "set.h"
#include "setup.h"
#include "traffic.h"
#include "node.h"



char *node_get(char **request, int args)
{
	char *output = NULL;

	if(args < 2)
		return strdup("ERROR: Operand missing\n");
	
	if(strcmp(request[1], "INTERFACES") == 0)
	{
		output = node_get_interfaces();
	}
	else
	if(strcmp(request[1], "IMAGES") == 0)
	{
		output = node_get_images();
	}

	return output;
}


char *vserver_control(char **request, int args)
{

	char *output = NULL;

	if(args < 3)
		return strdup("ERROR: Operand missing\n");
	if(!vserver_exist(request[2]))
		return strdup("ERROR: VServer does not exist\n");
	if(!vserver_lock(request[2]))
		return strdup("ERROR: VServer currently locked\n");

	if(strcmp(request[1], "START") == 0)
	{
		#ifdef DEBUG
			printf("START : [%s]\n", request[2]);
		#endif

		output = exec_cmd("vserver", 30, "vserver", request[2], "start", NULL);

	}
	else
	if(strcmp(request[1], "STOP") == 0)
	{
		#ifdef DEBUG
			printf("STOP : [%s]\n", request[2]);
		#endif
		
		output = exec_cmd("vserver", 30, "vserver", request[2], "stop", NULL);
	
	}


	vserver_unlock(request[2]);

	return output;
}

char *vserver_list(char **request, int args)
{
	char *output = NULL;
	int output_len = 0;
	DIR *dir_ptr;
	struct dirent *dir_entry;
	char *path;

	
      if((dir_ptr = opendir(global_settings.config_dir)) != NULL)
	{
		
		while((dir_entry = readdir(dir_ptr)) != NULL)
		{
			

			if(dir_entry->d_name[0] != '.')
			{
				path = concat(global_settings.config_dir, "/", dir_entry->d_name, NULL);

				if(dir_exist(path))
				{
					output = (char *)realloc(output, output_len + strlen(dir_entry->d_name) + 2);
					strcpy(output+output_len, dir_entry->d_name);

					output_len += strlen(dir_entry->d_name);
	
					output[output_len++] = ' ';
					output[output_len] = 0;
				}

				free(path);
			}
	
		}


		closedir(dir_ptr);
	}

	if(output != NULL)
		output[output_len-1] = '\n';
	else
		output = strdup("\n");


	return output;
}


char *vserver_set(char **request, int args)
{
	char *output = NULL;


	if(strcmp(request[1], "RLIMIT") == 0)
	{
		if(args < 5)
			return NULL;
		output = set_rlimit(request[2], request[3], request[4]);
	}
	else
	if(strcmp(request[1], "FLAG_ADD") == 0)
	{
		if(args < 4)
			return NULL;
		output = set_flag_add(request[2], request[3]);
	}
	else
	if(strcmp(request[1], "FLAG_REMOVE") == 0)
	{
		if(args < 4)
			return NULL;
		output = set_flag_remove(request[2], request[3]);
	}
	else
	if(strcmp(request[1], "SCHEDULE") == 0)
	{
		if(args < 4)
			return NULL;
		output = set_schedule(request[2], request[3]);
	}
	else
	if(strcmp(request[1], "IP_ADD") == 0)
	{
		if(args < 8)
			return NULL;
		output = add_ip(request[2], request[3], request[4], request[5], request[6], request[7]);
	}
	else
	if(strcmp(request[1], "IP_REMOVE") == 0)
	{
		if(args < 4)
			return NULL;
		output = remove_ip(request[2], request[3]);
	}
	else
	if(strcmp(request[1], "SPACE") == 0)
	{
		if(args < 5)
			return NULL;
		output = set_space(request[2], request[3], request[4]);
	}
	else
	if(strcmp(request[1], "HOSTNAME") == 0)
	{	
		if(args < 3)
			return NULL;
		output = set_hostname(request[2], request[3]);
	}
	else
	if(strcmp(request[1], "ROOTPASS") == 0)
	{
		if(args < 3)
			return NULL;
		output = set_rootpw(request[2], request[3]);
	}

	return output;
}


char *vserver_get(char **request, int args)
{
	char *output = NULL;

	

	if(!vserver_exist(request[2]))
			return strdup("ERROR: VServer does not exist\n");

	if(args < 3)
		return strdup("ERROR: Operand missing\n");


	if(strcmp(request[1], "XID") == 0)
	{

		output = get_xid(request[2]);

	}
	else
	if(strcmp(request[1], "STATE") == 0)
	{

		output = get_state(request[2]);

	}
	else
	if(strcmp(request[1], "INFO") == 0)
	{

		output = get_infos(request[2]);

	}
	else
	if(strcmp(request[1], "IPS") == 0)
	{

		output = get_ips(request[2]);

	}
	else
	if(strcmp(request[1], "HOSTNAME") == 0)
	{

		output = get_hostname(request[2]);

	}	
	else
	if(strcmp(request[1], "FLAGS") == 0)
	{

		output = get_flags(request[2]);

	}
	else
	if(strcmp(request[1], "SCHEDULE") == 0)
	{

		output = get_schedule(request[2]);

	}	
	else
	if(strcmp(request[1], "SPACE") == 0)
	{

		output = get_space(request[2]);

	}				
	else
	if(strcmp(request[1], "RLIMIT") == 0)
	{

		output = get_rlimit(request[2], request[3]);

	}

	return output;

}




char *vserver_traffic(char **request, int args)
{
	char buf[1024*10];
	char *output = NULL;
	double total = 0;
	struct traffic_st *traffic;

	int i = 0, entrys = 0;
	int output_len = 0;

	if(args < 3)
		return strdup("ERROR: Operand missing\n");

	if(strcmp(request[1], "ALL") == 0)
	{
		if(request[2] == NULL)
				return NULL;

		total = parse_server_traffic(request[2]);
					
		snprintf(buf, 1024*10-1, "%.0lf\n", total);
		output = strdup(buf);

	}	
	else
	{

		if(!vserver_exist(request[1]))
				return strdup("ERROR: VServer does not exist\n");
	
		traffic = parse_vserver_traffic(request[1], atoi(request[2]), &total, &entrys);
		
		if(traffic != NULL)
		{
			if(args == 4 && strcmp(request[3], "TOTAL") == 0)
			{
				snprintf(buf, 1024*10-1, "%.0lf\n", total);
				output = strdup(buf);
			}
			else
			{
				while(i != entrys)
				{
			
					snprintf(buf, 1024*10-1, "%s %d %d %.0lf %.0lf %.0lf ", traffic[i].ip, traffic[i].day, traffic[i].month, traffic[i].in, traffic[i].out, traffic[i].total);
					output = (char *)realloc(output, output_len+strlen(buf)+1);
					strcpy(output+output_len, buf);
		
					output_len += strlen(buf);
					i++;
				}	

				#ifdef DEBUG
					printf("TRAFFIC LEN: %d\n", output_len);	
				#endif

				snprintf(buf, 1024*10-1, "- %.0lf\n", total);
				output = (char *)realloc(output, output_len+strlen(buf)+1);
				strcpy(output+output_len, buf);
			}		

			free(traffic);
		}
		else
		{
			output = strdup("N/A\n");
		}

	}

	return output;

}

char *vserver_setup(char **request, int args)
{

	char *output;


	if(strcmp(request[1], "CONFIG") == 0)
	{
		if(args < 3)
			return strdup("ERROR: Operand missing\n");

		output = setup_config(request[2]);

	}
	else
	if(strcmp(request[1], "IMAGE") == 0)
	{

		if(args < 4)
			return strdup("ERROR: Operand missing\n");

		if(!vserver_lock(request[2]))
			return strdup("ERROR: VServer currently locked\n");

		output = setup_image(request[2], request[3]);

		vserver_unlock(request[2]);
	}	
	


	return output;
}

char *vserver_remove(char **request, int args)
{

	char *output,  *path;
	
	if(args < 2)
		return strdup("ERROR: Operand missing\n");
	
	if(request[1] != NULL)
	{

		path = concat(global_settings.config_dir, "/", request[1], NULL);

		if(dir_exist(path))
		{
			output = exec_cmd("rm", 10, "rm", "-rf", path, NULL);
			free(output);
		}
		
		free(path);
		path = concat(global_settings.root_dir, "/", request[1], NULL);
		
		if(dir_exist(path))
		{
			output = exec_cmd("rm", 180, "rm", "-rf", path, NULL);
			free(output);
		}

		free(path);	
		
		output = strdup("OK\n");
	}
	else
	{
		output = strdup("ERROR\n");
	}


	return output;
}

