/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdarg.h>
#include <fcntl.h>

#include "fs.h"
#include "set.h"
#include "main.h"
#include "config.h"
#include "strings.h"

char *set_rlimit(char *name, char *limit, char *value)
{
	char *filepath;
	char *dir;
	char *output = NULL;
	
	dir = concat(global_settings.config_dir, "/", name, "/rlimits", NULL);
	filepath = concat(dir, "/", limit, NULL);

	if(!dir_exist(dir))
	{
		mkdir(dir, 493); // rwxr-xr-x
	}
	
	if(strcmp(value, "0") == 0)
	{
		if (file_exist(filepath))
			unlink(filepath);
	}
	else
	{
		file_write(filepath, value); 
		file_attach(filepath, "\n");
	}
	
	output = strdup("OK\n");
	
	free(filepath);
	return output;
}

char *set_flag_add(char *name, char *flag)
{
	char *filepath;
	char *output = NULL;
	
	filepath = concat(global_settings.config_dir, "/", name, "/flags", NULL);
	
	file_attach(filepath, flag); 	
			
	file_attach(filepath, "\n");

	output = strdup("OK\n");
	
	free(filepath);
	return output;
}

char *set_flag_remove(char *name, char *flag)
{
	char *filepath, *flagfile, *p;
	char *output = NULL;
	int flagfile_fd;

	filepath = concat(global_settings.config_dir, "/", name, "/flags", NULL);
	flagfile = file_read(filepath);
	
	flagfile_fd = open(filepath, O_CREAT | O_TRUNC | O_WRONLY);

	if(flagfile_fd != -1 && flagfile != NULL)
	{	
		p = strstr(flagfile, flag);
		if(p != NULL)
		{
			 write(flagfile_fd, flagfile, p-flagfile);
			 write(flagfile_fd, p+strlen(flag)+1, strlen(p+strlen(flag)+1));
		}
		else
		{
			 write(flagfile_fd, flagfile, strlen(flagfile));
		}

		output = strdup("OK\n");
	}
	else
	{
		output = strdup("ERROR\n");
	}

	
	if(flagfile != NULL) free(flagfile);
	free(filepath);
	return output; 
}

char *set_schedule(char *name, char *percent)
{
	char  *output = NULL;
	char *filepath, *string;

	#ifdef DEBUG
	 	printf("LIMIT: [%s]]\n\n", percent);
   	#endif

	filepath = concat(global_settings.config_dir, "/", name, "/schedule", NULL);
	string = concat(percent, "\n32\n500\n200\n1000\ndummy\n", NULL);
			
	file_write(filepath, string);
	
	
	output = strdup("OK\n");

	free(filepath); free(string);

	return output;
}



char *add_ip(char *name, char *ip, char *device, char *alias, char *netmask, char *bcast)
{
	int i = 0, ret = 1;
	char path[BUF_SIZE+1], tmppath[BUF_SIZE+1];
	char *output = NULL;
	
	memset(path, 0, BUF_SIZE+1);
	
   	while(ret)
	{		
		snprintf(path, 500, "%s/%s/interfaces/%d", global_settings.config_dir, name, i);
		ret = dir_exist(path);
      		i++;
	}


	if(mkdir(path, 0755) != -1)
	{

		if(strcmp(alias, "noname") != 0)
		{	
			sconcat(tmppath, BUF_SIZE, path, "/name", NULL);
			file_write(tmppath, alias); file_attach(tmppath, "\n");
		}
	
		if(strcmp(device, "nodev") == 0)
		{
			sconcat(tmppath, BUF_SIZE, path, "/nodev", NULL);
		}
		else
		{
			sconcat(tmppath, BUF_SIZE, path, "/dev", NULL);
		}
		file_write(tmppath, device); file_attach(tmppath, "\n");


		if(strcmp(netmask, "nomask") != 0)
		{
			sconcat(tmppath, BUF_SIZE, path, "/mask", NULL);
			file_write(tmppath, netmask); file_attach(tmppath, "\n");
		}


		if(strcmp(bcast, "nobcast") != 0)
		{
			sconcat(tmppath, BUF_SIZE, path, "/bcast", NULL);
			file_write(tmppath, bcast); file_attach(tmppath, "\n");
		}



		sconcat(tmppath, BUF_SIZE, path, "/ip", NULL);
		file_write(tmppath, ip); file_attach(tmppath, "\n");

	
		output = strdup("OK\n");
	
	}


	return output;
}


char *remove_ip(char *name, char *delip)
{
	char *output = NULL;
	char path[BUF_SIZE+1], *ip;
	
	struct dirent *dir_info;
	DIR *dirst;

	memset(path, 0, BUF_SIZE+1);
	
	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces", NULL);
	dirst = opendir(path);
	
	while ((dir_info = readdir(dirst)) != NULL)
	{
	
		if(dir_info->d_name[0] != '.')
		{
	
			sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/",  dir_info->d_name, NULL);
	
			if(dir_exist(path))
			{
				sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/",  dir_info->d_name, "/ip", NULL);
	
				ip = file_read(path);
				trim_string(ip);
			
				#ifdef DEBUG
					printf("Compare: [%s]/[%s]\n", ip, delip);	
				#endif

				if(strcmp(ip, delip) == 0)
				{
					#ifdef DEBUG
						printf("Deleting: %s", ip);
					#endif 

					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/",  dir_info->d_name, NULL);
					dir_remove(path);
	
				
					free(ip);
					break;
				}
				               
				free(ip);
				
			}
		
		}
	}
	
	closedir(dirst);
	
	output = strdup("OK\n");
	
	return output;
   
   
}

char *set_space(char *name, char *space, char *inodes)
{
	char path[BUF_SIZE+1], *data;
	char *output;

	memset(path, 0, BUF_SIZE+1);

	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/dlimits", NULL);

	if(!dir_exist(path))
		mkdir(path, 0755);


	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/dlimits/0", NULL);

	if(!dir_exist(path))
		mkdir(path, 0755);


        sconcat(path, BUF_SIZE,global_settings.config_dir, "/", name, "/dlimits/0/directory", NULL);
	data = concat(global_settings.root_dir, "/", name, "\n", NULL);
	file_write(path, data); file_attach(path, "\n");
	free(data);	

	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/dlimits/0/space_total", NULL);
	file_write(path, space); file_attach(path, "\n");

	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/dlimits/0/inodes_total", NULL);
	file_write(path, inodes); file_attach(path, "\n");

	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/dlimits/0/reserved", NULL);
	file_write(path, "5\n");
	


	output = strdup("OK\n");
	

	return output;
}

char *set_rootpw(char *name, char *pw)
{
	char *path;
	char *output = NULL;
	
	path = concat(global_settings.root_dir, "/", name, "/etc/shadow", NULL);
	
	if(file_exist(path))
	{
		file_replace(path, "#ROOTPASS#", pw);

		output = strdup("OK\n");
	}
	else
	{
		output = strdup("Failed\n");
	}

	free(path);

	return output; 
}

char *set_hostname(char *name, char *hostname)
{
	char path[BUF_SIZE+1], subhostname[BUF_SIZE+1];
	char *ip, *hosts;
	char *output = NULL;
	int i = 0, len;

	memset(path, 0, BUF_SIZE+1);	

	len = strlen(hostname);

	while(hostname[i] != '.' && i < len && i < BUF_SIZE)
	{
		subhostname[i] = hostname[i];
		i++;
	}

	subhostname[i] = 0;


	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/0/ip", NULL);
	ip = file_read(path);

	if(ip != NULL)
	{
		sconcat(path, BUF_SIZE, global_settings.root_dir, "/", name, "/etc/hosts", NULL);
		trim_string(ip);

		if(i != len)
		{
			hosts = concat(ip, "\t", subhostname, "\t", hostname,"\n", NULL);
		}
		else
		{
			hosts = concat(ip, "\t", hostname,"\n", NULL);
		}

		file_write(path, hosts);

		free(ip); free(hosts);
	}

	sconcat(path, BUF_SIZE, global_settings.root_dir, "/", name,"/etc/hostname", NULL);
	file_write(path, subhostname); file_attach(path, "\n");

	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name,"/uts/nodename", NULL);
	file_write(path, subhostname); file_attach(path, "\n");


	output = strdup("OK\n");


	return output;
}

