/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "config.h"
#include "main.h"
#include "setup.h"
#include "fs.h"
#include "strings.h"
#include "misc.h"

char *setup_config(char *name)
{
	char path1[BUF_SIZE+1], path2[BUF_SIZE+1];
	char *configdir, *xid;
	int xid_int;
	char *output = NULL;

	memset(path1, 0, BUF_SIZE+1);
	memset(path2, 0, BUF_SIZE+1);

	configdir = concat(global_settings.config_dir, "/", name, NULL);
	
	if(mkdir(configdir, 0755) != -1)
	{

		/* Directorys */	
		sconcat(path1, BUF_SIZE, configdir, "/interfaces", NULL);
		mkdir(path1, 0755);

		sconcat(path1, BUF_SIZE, configdir, "/uts", NULL);
		mkdir(path1, 0755);

		sconcat(path1, BUF_SIZE, configdir, "/name", NULL);
		file_write(path1, name); file_attach(path1, "\n");
	

		/* Files */
		xid = file_read(XID_COUNTER);

		if(xid != NULL)
		{
			trim_string(xid);
			xid_int = atoi(xid);

			xid_int += 1;
			free(xid);
	
			xid = (char *)malloc(BUF_SIZE+1);
			snprintf(xid, BUF_SIZE, "%d", xid_int);
			file_write(XID_COUNTER, xid);
		}
		else
		{
			xid = strdup(START_XID);
			file_write(XID_COUNTER, xid);
		}

		

		sconcat(path1, BUF_SIZE, configdir, "/context", NULL);
		file_write(path1, xid); file_attach(path1, "\n");
		free(xid);


		sconcat(path1, BUF_SIZE, configdir, "/fstab", NULL);
		file_write(path1, "none /proc proc defaults 0 0\nnone /dev/pts devpts gid=5,mode=620 0 0\n");
	
		/* Symlinks */
		sconcat(path1, BUF_SIZE, global_settings.run_dir, "/", name, NULL);
		sconcat(path2, BUF_SIZE, configdir, "/run", NULL);
		symlink(path1, path2);
	
		sconcat(path1, BUF_SIZE, global_settings.root_dir, "/", name, NULL);
		sconcat(path2, BUF_SIZE, configdir, "/vdir", NULL);
		symlink(path1, path2);

		sconcat(path1, BUF_SIZE, global_settings.cache_dir, "/", name, NULL);
		sconcat(path2, BUF_SIZE, configdir, "/cache", NULL);
		symlink(path1, path2);


		output = strdup("OK\n");
	}
	
	free(configdir);
	

	
	return output;
}

char *setup_image(char *name, char *dist)
{
	char path[BUF_SIZE+1];
	char *output = NULL;
	char *context, *homedir, *imagedir, *backupdir;
	
	memset(path, 0, BUF_SIZE+1);

	if(name != NULL && dist != NULL)
	{
		backupdir = concat(global_settings.backup_dir, "/", name, NULL);
		imagedir = concat(global_settings.image_dir, "/", dist, NULL);
		homedir = concat(global_settings.root_dir, "/", name, NULL);

		if(!dir_exist(imagedir))
			return NULL;


		if(dir_exist(homedir))
		{

			sconcat(path, BUF_SIZE, homedir, "/backup", NULL);	
			
			if(dir_exist(path))
			{
					
				if(dir_exist(global_settings.backup_dir))
				{
					output = exec_cmd("mv", 900, "mv", path, backupdir, NULL);
					if(output != NULL) free(output);
				}
			}

			output = exec_cmd("rm", 900, "rm", "-rf", homedir, NULL);
			if(output != NULL) free(output);
		
		}



		output = exec_cmd("cp", 900, "cp", "-a", imagedir, homedir, NULL);
		if(output != NULL) free(output);
			
		if(dir_exist(backupdir))
		{
			sconcat(path, BUF_SIZE, homedir, "/backup", NULL);	
			output = exec_cmd("mv", 900, "mv", backupdir, path, NULL);
			if(output != NULL) free(output);
		}
		
		sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/context", NULL);	
		context = file_read(path);	
		trim_string(context);
	
		output = exec_cmd("chxid", 900, "chxid", "-c",  context, "-R", homedir, NULL);
		if(output != NULL) free(output);
		
		free(context);



		sconcat(path, BUF_SIZE, global_settings.image_dir, "/", dist, ".sh", NULL);
	
	        if(file_exist(path))
                {
                      printf("Executing: %s\n", path);
		      output = exec_cmd("sh", 900, "sh", path, name, NULL);
                      if(output != NULL) free(output);

                }
		
		output = strdup("OK\n");

		free(backupdir);
		free(imagedir);
		free(homedir);

	}
	else
	{
		output = strdup("ERROR\n");
	
	}
	
		


	return output;

}

