/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>
#include <unistd.h>

#include "traffic.h"
#include "main.h"
#include "fs.h"
#include "misc.h"



struct traffic_st *parse_vserver_traffic(char *name, int month, double *total_month, int *total_entrys)
{
	char *ip, **ips, buf[1024]; 
	double in, out, total; 
	struct traffic_st *output = NULL;
	int i = 1, ip_num, ips_num;
	int entry_num = 0;

	FILE *logfile;
	
	*total_month = 0;
	
	ips = parse_ips(name, &ips_num);

	if(ips == NULL)
		return NULL;

	while(i <= 31)
	{

		snprintf(buf, 1024-1, "%s/%02d/%02d", global_settings.ipfm_dir, month, i);

		if(file_exist(buf))
		{
			#ifdef DEBUG
				//printf("Traffic File check: %s\n", buf);
			#endif

			logfile = fopen(buf, "r");


			while(fgets(buf, 1024-1, logfile) != NULL)
			{

				if(buf[0] != '#')
				{
		

					sscanf(buf, "%as %lf %lf %lf", &ip, &in, &out, &total);
					
					ip_num = 0;

					while(ip_num <= ips_num-1)	
					{
						#ifdef DEBUG		
							//printf("Traffic check IP: %d/%d [%s]\n", ip_num, ips_num, ips[ip_num]);
						#endif

						if(strcmp(ip, ips[ip_num]) == 0)
						{		
							output = (struct traffic_st *)realloc(output, sizeof(struct traffic_st) * (entry_num+1));

	
							output[entry_num].day = i;						
							output[entry_num].month = month;						
	
							strncpy(output[entry_num].ip, ip, 20);
							output[entry_num].in = in;
							output[entry_num].out = out;
							output[entry_num].total = total;						
		
							*total_month += total;	

							entry_num++;
	
							break;
						}	
	
						ip_num++;
					}	

					free(ip);
	
				}
			
			}

			fclose(logfile);

		}
		
		i++;
	}

	*total_entrys = entry_num;

	free_strings(ips, ips_num);
	
	return output;
}



double parse_server_traffic(char *month)
{
	struct dirent *dir_entry;
	double total = 0, total_vs;
	int total_entrys;
	DIR *vsdir;


	struct traffic_st *traffic;


	vsdir = opendir(global_settings.config_dir);
	
	while((dir_entry = readdir(vsdir)) != NULL)
	{
		if(dir_entry->d_name[0] != '.')
		{
			traffic = parse_vserver_traffic(dir_entry->d_name, atoi(month), &total_vs, &total_entrys);
			total += total_vs;		
			
			free(traffic);
		}
		
	}
	
	closedir(vsdir);
	
	return total;
}


